@@ -48,7 +48,7 @@ Caching:
         findimports.py foo.importcache -d -N -c -p -l 2 > graph2.dot
 
 
-Copyright (c) 2003--2008 Marius Gedminas <marius@pov.lt>
+Copyright (c) 2003--2009 Marius Gedminas <marius@pov.lt>
 
 This program is free software; you can redistribute it and/or modify it under
 the terms of the GNU General Public License as published by the Free Software
@@ -66,18 +66,23 @@ Ave, Cambridge, MA 02139, USA.
 
 import os
 import sys
-import sets
 import getopt
 import doctest
 import compiler
 import linecache
 import pickle
+import zipfile
 from operator import attrgetter
-from compiler import ast
 from compiler.visitor import ASTVisitor
 
+try:
+    set
+except NameError:
+    # Python 2.4 compatibility
+    from sets import Set as set
 
-__version__ = '1.2.7'
+
+__version__ = '1.2.8'
 
 
 def adjust_lineno(filename, lineno, name):
@@ -334,7 +339,7 @@ class Module(object):
         self.modname = modname
         self.label = modname
         self.filename = filename
-        self.imports = sets.Set()
+        self.imports = set()
         self.imported_names = ()
         self.unused_names = ()
 
@@ -349,7 +354,7 @@ class ModuleCycle(object):
         self.modnames = modnames
         self.modname = modnames[0]
         self.label = "\n".join(modnames)
-        self.imports = sets.Set()
+        self.imports = set()
 
 
 class ModuleGraph(object):
@@ -367,7 +372,7 @@ class ModuleGraph(object):
         self._module_cache = {}
         # some builtin modules cannot be found using normal means, but we don't
         # want to report warnings about them
-        self._warned_about = sets.Set(['gc'])
+        self._warned_about = set(['gc'])
 
     def parsePathname(self, pathname):
         """Parse one or more source files.
@@ -413,7 +418,7 @@ class ModuleGraph(object):
             module.imported_names = find_imports(filename)
             module.unused_names = None
         dir = os.path.dirname(filename)
-        module.imports = sets.Set(
+        module.imports = set(
             [self.findModuleOfName(imp.name, filename, dir)
              for imp in module.imported_names])
 
@@ -478,13 +483,24 @@ class ModuleGraph(object):
         except KeyError:
             pass
         for dir in self.path:
-            for ext in ('.py', '.so', '.dll'):
-                candidate = os.path.join(dir, filename) + ext
-                if os.path.exists(candidate):
-                    modname = self.filenameToModname(candidate)
-                    self._module_cache[(dotted_name, extrapath)] = modname
-                    self._module_cache[(dotted_name, None)] = modname
-                    return modname
+            if os.path.isfile(dir):
+                zf = zipfile.ZipFile(dir)
+                names = zf.namelist()
+                for ext in ('.py', '.so', '.dll'):
+                    candidate = filename + ext
+                    if candidate in names:
+                        modname = filename.replace(os.path.sep, '.')
+                        self._module_cache[(dotted_name, extrapath)] = modname
+                        self._module_cache[(dotted_name, None)] = modname
+                        return modname
+            else:
+                for ext in ('.py', '.so', '.dll'):
+                    candidate = os.path.join(dir, filename) + ext
+                    if os.path.exists(candidate):
+                        modname = self.filenameToModname(candidate)
+                        self._module_cache[(dotted_name, extrapath)] = modname
+                        self._module_cache[(dotted_name, None)] = modname
+                        return modname
         return None
 
     def isPackage(self, dotted_name, extrapath=None):
@@ -566,7 +582,7 @@ class ModuleGraph(object):
         # Phase 0: prepare the graph
         imports = {}
         for u in self.modules:
-            imports[u] = sets.Set()
+            imports[u] = set()
             for v in self.modules[u].imports:
                 if v in self.modules: # skip external dependencies
                     imports[u].add(v)
@@ -588,7 +604,7 @@ class ModuleGraph(object):
         # Phase 2: compute the inverse graph
         revimports = {}
         for u in self.modules:
-            revimports[u] = sets.Set()
+            revimports[u] = set()
         for u in self.modules:
             for v in imports[u]:
                 revimports[v].add(u)
@@ -659,7 +675,7 @@ class ModuleGraph(object):
         """Produce a dependency graph in dot format."""
         print "digraph ModuleDependencies {"
         print "  node[shape=box];"
-        allNames = sets.Set()
+        allNames = set()
         nameDict = {}
         for n, module in enumerate(self.listModules()):
             module._dot_name = "mod%d" % n
@@ -669,7 +685,7 @@ class ModuleGraph(object):
             allNames |= module.imports
         print "  node[style=dotted];"
         if self.external_dependencies:
-            myNames = sets.Set(self.modules)
+            myNames = set(self.modules)
             extNames = list(allNames - myNames)
             extNames.sort()
             for n, name in enumerate(extNames):
